<?php

namespace Zyimm\Sms\Drivers;

use Hyperf\Utils\Arr;
use Zyimm\Sms\Contracts\SmsableInterface;
use Zyimm\Sms\Exceptions\DriverErrorException;

class HogeDriver extends AbstractDriver
{
    const SEND_URL = '/api/sms/send';

    const USE_CASE_URL = '/api/sms/usecase';

    public function send(SmsableInterface $smsable): array
    {
        $url                  = $this->buildUrl(self::SEND_URL);
        $data                 = [
            'mobile' => $smsable->to->getNationalNumber()
        ];
        $data['custom_appid'] = $this->config->get('sms_appid');
        $data['client_ip']    = $this->config->get('client_ip', '127.0.0.1');
        $data['sign_name']    = $this->config->get('sms_sign');
        $data['timestamp']    = time();
        $data['content']      = $smsable->content;
        $string_temp          = $this->getSignContent(Arr::except($data,
                'signature')).'&key='.$this->config->get('sms_appkey');
        $signature            = strtoupper(hash_hmac('sha256', $string_temp, $this->config->get('sms_appkey')));
        $data['signature']    = $signature;
        $url                  = $url.'?'.http_build_query($data);
        $response             = $this->client->post($url, $data);
        $result               = $response->toArray() ?? [];
        if ($result['code'] != 200) {
            throw new DriverErrorException($result['msg'] ?? '未知错误', $result['code'], $response);
        }
        return [];
    }

    protected function getSignContent($content): string
    {
        ksort($content);
        $sign_data = [];
        foreach ($content as $k => $v) {
            if (is_array($v)) {
                $v = json_encode($v);
            }
            $v = trim($v);
            if (!empty($v)) {
                $sign_data[] = $k.'='.$v;
            }
        }
        return implode('&', $sign_data);
    }

    /**
     * buildUrl
     *
     * @param  string  $path
     * @return string
     */
    protected function buildUrl(string $path): string
    {
        return $this->config->get('sms_api_url').$path;
    }

    public function serivce()
    {
        $url = $this->config->get('sms_api_url').self::USE_CASE_URL;

        $response = $this->client->get($url, [
            'custom_appid'  => $this->config->get('sms_appid'),
            'custom_appkey' => $this->config->get('sms_appkey')
        ]);
        $result =  $response->toArray();
        if ($result['code'] != 200) {
            throw new DriverErrorException($result['message'] ?? '未知错误', $result['code'], $response);
        }
        return $result['data'] ?? [];
    }
}